@@ -6,8 +6,11 @@ import logging
 import os
 import tempfile
 from collections.abc import Sequence
+from pathlib import Path
+from typing import Any
 
 import pytest
+import pytest_mock
 import responses
 
 import tldextract
@@ -62,34 +65,40 @@ def assert_extract(
         assert expected_ipv6_data == ext.ipv6
 
 
-def test_american():
+def test_american() -> None:
+    """Test a common suffix, .com."""
     assert_extract("http://www.google.com", ("www.google.com", "www", "google", "com"))
 
 
-def test_british():
+def test_british() -> None:
+    """Test a British suffix."""
     assert_extract(
         "http://www.theregister.co.uk",
         ("www.theregister.co.uk", "www", "theregister", "co.uk"),
     )
 
 
-def test_no_subdomain():
+def test_no_subdomain() -> None:
+    """Test no explicit subdomain."""
     assert_extract("http://gmail.com", ("gmail.com", "", "gmail", "com"))
 
 
-def test_nested_subdomain():
+def test_nested_subdomain() -> None:
+    """Test multiple levels of subdomain."""
     assert_extract(
         "http://media.forums.theregister.co.uk",
         ("media.forums.theregister.co.uk", "media.forums", "theregister", "co.uk"),
     )
 
 
-def test_odd_but_possible():
+def test_odd_but_possible() -> None:
+    """Test an odd but possible use of the common .com suffix."""
     assert_extract("http://www.www.com", ("www.www.com", "www", "www", "com"))
     assert_extract("http://www.com", ("www.com", "", "www", "com"))
 
 
-def test_suffix():
+def test_suffix() -> None:
+    """Test strings that only contain a suffix, until the point they contain more components."""
     assert_extract("com", ("", "", "", "com"))
     assert_extract("co.uk", ("", "", "", "co.uk"))
     assert_extract("example.ck", ("", "", "", "example.ck"))
@@ -102,7 +111,8 @@ def test_suffix():
     assert_extract("buskerud.no", ("buskerud.no", "", "buskerud", "no"))
 
 
-def test_local_host():
+def test_local_host() -> None:
+    """Test local hostnames, i.e. no recognized public suffix."""
     assert_extract(
         "http://internalunlikelyhostname/", ("", "", "internalunlikelyhostname", "")
     )
@@ -110,9 +120,6 @@ def test_local_host():
         "http://internalunlikelyhostname.bizarre",
         ("", "internalunlikelyhostname", "bizarre", ""),
     )
-
-
-def test_qualified_local_host():
     assert_extract(
         "http://internalunlikelyhostname.info/",
         ("internalunlikelyhostname.info", "", "internalunlikelyhostname", "info"),
@@ -123,19 +130,8 @@ def test_qualified_local_host():
     )
 
 
-def test_ip():
-    assert_extract(
-        "http://216.22.0.192/",
-        ("", "", "216.22.0.192", ""),
-        expected_ip_data="216.22.0.192",
-    )
-    assert_extract(
-        "http://216.22.project.coop/",
-        ("216.22.project.coop", "216.22", "project", "coop"),
-    )
-
-
-def test_lenient_netloc():
+def test_lenient_netloc() -> None:
+    """Test function to leniently extract the netloc from a URL."""
     assert lenient_netloc("https://example.com.ca") == "example.com.ca"
     assert lenient_netloc("https://[example.com.ca]") == "[example.com.ca]"
     assert lenient_netloc("https://[example.com.ca]:5000") == "[example.com.ca]"
@@ -156,27 +152,31 @@ def test_lenient_netloc():
 
 
 @pytest.mark.skipif(not inet_pton, reason="inet_pton unavailable")
-def test_looks_like_ip_with_inet_pton():
+def test_looks_like_ip_with_inet_pton() -> None:
+    """Test preferred function to check if a string looks like an IP address."""
     assert looks_like_ip("1.1.1.1", inet_pton) is True
     assert looks_like_ip("a.1.1.1", inet_pton) is False
     assert looks_like_ip("1.1.1.1\n", inet_pton) is False
     assert looks_like_ip("256.256.256.256", inet_pton) is False
 
 
-def test_looks_like_ip_without_inet_pton():
+def test_looks_like_ip_without_inet_pton() -> None:
+    """Test fallback function to check if a string looks like an IP address."""
     assert looks_like_ip("1.1.1.1", None) is True
     assert looks_like_ip("a.1.1.1", None) is False
     assert looks_like_ip("1.1.1.1\n", None) is False
     assert looks_like_ip("256.256.256.256", None) is False
 
 
-def test_similar_to_ip():
+def test_similar_to_ip() -> None:
+    """Test strings that look like IP addresses but are not."""
     assert_extract("1\xe9", ("", "", "1\xe9", ""))
     assert_extract("1.1.1.1\ncom", ("", "1.1.1", "1\ncom", ""))
     assert_extract("1.1.1.1\rcom", ("", "1.1.1", "1\rcom", ""))
 
 
-def test_punycode():
+def test_punycode() -> None:
+    """Test URLs with Punycode."""
     assert_extract(
         "http://xn--h1alffa9f.xn--p1ai",
         ("xn--h1alffa9f.xn--p1ai", "", "xn--h1alffa9f", "xn--p1ai"),
@@ -212,7 +212,8 @@ def test_punycode():
     )
 
 
-def test_invalid_puny_with_puny():
+def test_invalid_puny_with_puny() -> None:
+    """Test URLs with a mix of in/valid Punycode."""
     assert_extract(
         "http://xn--zckzap6140b352by.blog.so-net.xn--wcvs22d.hk",
         (
@@ -227,11 +228,13 @@ def test_invalid_puny_with_puny():
     )
 
 
-def test_invalid_puny_with_nonpuny():
+def test_invalid_puny_with_nonpuny() -> None:
+    """Test URLs with a mix of invalid Punycode and non-Punycode."""
     assert_extract("xn--ß‌꫶ᢥ.com", ("xn--ß‌꫶ᢥ.com", "", "xn--ß‌꫶ᢥ", "com"))
 
 
-def test_puny_with_non_puny():
+def test_puny_with_non_puny() -> None:
+    """Test URLs with a mix of in/valid Punycode."""
     assert_extract(
         "http://xn--zckzap6140b352by.blog.so-net.教育.hk",
         (
@@ -243,8 +246,12 @@ def test_puny_with_non_puny():
     )
 
 
-def test_idna_2008():
-    """Python supports IDNA 2003. The IDNA library adds 2008 support for characters like ß."""
+def test_idna_2008() -> None:
+    """Test that this project relies on the IDNA library.
+
+    Python's standard library supports IDNA 2003. The IDNA 3rd party library
+    adds 2008 support for characters like ß.
+    """
     assert_extract(
         "xn--gieen46ers-73a.de",
         ("xn--gieen46ers-73a.de", "", "xn--gieen46ers-73a", "de"),
@@ -255,11 +262,13 @@ def test_idna_2008():
     )
 
 
-def test_empty():
+def test_empty() -> None:
+    """Test an empty URL."""
     assert_extract("http://", ("", "", "", ""))
 
 
-def test_scheme():
+def test_scheme() -> None:
+    """Test a mix of in/valid schemes."""
     assert_extract("//", ("", "", "", ""))
     assert_extract("://", ("", "", "", ""))
     assert_extract("://example.com", ("", "", "", ""))
@@ -283,13 +292,15 @@ def test_scheme():
     )
 
 
-def test_port():
+def test_port() -> None:
+    """Test a URL with a port number."""
     assert_extract(
         "git+ssh://www.github.com:8443/", ("www.github.com", "www", "github", "com")
     )
 
 
-def test_username():
+def test_username() -> None:
+    """Test URLs with usernames and passwords."""
     assert_extract(
         "ftp://johndoe:5cr1p7k1dd13@1337.warez.com:2501",
         ("1337.warez.com", "1337", "warez", "com"),
@@ -311,7 +322,8 @@ def test_username():
     )
 
 
-def test_query_fragment():
+def test_query_fragment() -> None:
+    """Test URLs with query strings and fragments."""
     assert_extract("http://google.com?q=cats", ("google.com", "", "google", "com"))
     assert_extract("http://google.com#Welcome", ("google.com", "", "google", "com"))
     assert_extract("http://google.com/#Welcome", ("google.com", "", "google", "com"))
@@ -321,7 +333,8 @@ def test_query_fragment():
     )
 
 
-def test_regex_order():
+def test_order() -> None:
+    """Test that a more-specific suffix is preferred."""
     assert_extract(
         "http://www.parliament.uk", ("www.parliament.uk", "www", "parliament", "uk")
     )
@@ -331,25 +344,8 @@ def test_regex_order():
     )
 
 
-def test_unhandled_by_iana():
-    assert_extract(
-        "http://www.cgs.act.edu.au/", ("www.cgs.act.edu.au", "www", "cgs", "act.edu.au")
-    )
-    assert_extract(
-        "http://www.google.com.au/", ("www.google.com.au", "www", "google", "com.au")
-    )
-
-
-def test_tld_is_a_website_too():
-    assert_extract(
-        "http://www.metp.net.cn", ("www.metp.net.cn", "www", "metp", "net.cn")
-    )
-    # This is unhandled by the PSL. Or is it?
-    # assert_extract(http://www.net.cn',
-    #                ('www.net.cn', 'www', 'net', 'cn'))
-
-
-def test_no_1st_level_tld():
+def test_no_1st_level_tld() -> None:
+    """Test a suffix without a first level, .za."""
     assert_extract("za", ("", "", "za", ""))
     assert_extract("example.za", ("", "example", "za", ""))
     assert_extract("co.za", ("", "", "", "co.za"))
@@ -359,7 +355,8 @@ def test_no_1st_level_tld():
     )
 
 
-def test_dns_root_label():
+def test_dns_root_label() -> None:
+    """Test handling a fully qualified domain name, i.e. a DNS root label, i.e. a trailing dot."""
     assert_extract(
         "http://www.example.com./", ("www.example.com", "www", "example", "com")
     )
@@ -374,14 +371,17 @@ def test_dns_root_label():
     )
 
 
-def test_private_domains():
+def test_ipv4() -> None:
+    """Test IPv4 addresses."""
     assert_extract(
-        "http://waiterrant.blogspot.com",
-        ("waiterrant.blogspot.com", "waiterrant", "blogspot", "com"),
+        "http://216.22.0.192/",
+        ("", "", "216.22.0.192", ""),
+        expected_ip_data="216.22.0.192",
+    )
+    assert_extract(
+        "http://216.22.project.coop/",
+        ("216.22.project.coop", "216.22", "project", "coop"),
     )
-
-
-def test_ipv4():
     assert_extract(
         "http://127.0.0.1/foo/bar",
         ("", "", "127.0.0.1", ""),
@@ -394,15 +394,13 @@ def test_ipv4():
     )
 
 
-def test_ipv4_bad():
+def test_ipv4_lookalike() -> None:
+    """Test what looks like an IPv4 address, but isn't."""
     assert_extract(
         "http://256.256.256.256/foo/bar",
         ("", "256.256.256", "256", ""),
         expected_ip_data="",
     )
-
-
-def test_ipv4_lookalike():
     assert_extract(
         "http://127.0.0/foo/bar", ("", "127.0", "0", ""), expected_ip_data=""
     )
@@ -414,33 +412,22 @@ def test_ipv4_lookalike():
     )
 
 
-def test_result_as_dict():
-    result = extract(
-        "http://admin:password1@www.google.com:666/secret/admin/interface?param1=42"
-    )
-    expected_dict = {
-        "subdomain": "www",
-        "domain": "google",
-        "suffix": "com",
-        "is_private": False,
-    }
-    assert result._asdict() == expected_dict
-
-
-def test_cache_permission(mocker, monkeypatch, tmpdir):
-    """Emit a warning once that this can't cache the latest PSL."""
+def test_cache_permission(
+    mocker: pytest_mock.MockerFixture, monkeypatch: pytest.MonkeyPatch, tmp_path: Path
+) -> None:
+    """Emit a warning once when the library can't cache the latest PSL."""
     warning = mocker.patch.object(logging.getLogger("tldextract.cache"), "warning")
 
-    def no_permission_makedirs(*args, **kwargs):
+    def no_permission_makedirs(*args: Any, **kwargs: Any) -> None:
         raise PermissionError(
             """[Errno 13] Permission denied:
-            '/usr/local/lib/python3.7/site-packages/tldextract/.suffix_cache"""
+            '/usr/local/lib/python3.11/site-packages/tldextract/.suffix_cache"""
         )
 
     monkeypatch.setattr(os, "makedirs", no_permission_makedirs)
 
     for _ in range(0, 2):
-        my_extract = tldextract.TLDExtract(cache_dir=tmpdir)
+        my_extract = tldextract.TLDExtract(cache_dir=str(tmp_path))
         assert_extract(
             "http://www.google.com",
             ("www.google.com", "www", "google", "com"),
@@ -452,27 +439,33 @@ def test_cache_permission(mocker, monkeypatch, tmpdir):
 
 
 @responses.activate
-def test_cache_timeouts(tmpdir):
+def test_cache_timeouts(tmp_path: Path) -> None:
+    """Test error raised when all upstream servers time out."""
     server = "http://some-server.com"
     responses.add(responses.GET, server, status=408)
-    cache = DiskCache(tmpdir)
+    cache = DiskCache(str(tmp_path))
 
     with pytest.raises(SuffixListNotFound):
         tldextract.suffix_list.find_first_response(cache, [server], 5)
 
 
-def test_include_psl_private_domain_attr():
+def test_include_psl_private_domain_attr() -> None:
+    """Test private domains, which default to not being treated differently."""
     extract_private = tldextract.TLDExtract(include_psl_private_domains=True)
-    extract_public = tldextract.TLDExtract(include_psl_private_domains=False)
+    extract_public1 = tldextract.TLDExtract()
+    extract_public2 = tldextract.TLDExtract(include_psl_private_domains=False)
     assert extract_private("foo.uk.com") == ExtractResult(
         subdomain="", domain="foo", suffix="uk.com", is_private=True
     )
-    assert extract_public("foo.uk.com") == ExtractResult(
-        subdomain="foo", domain="uk", suffix="com", is_private=False
+    assert (
+        extract_public1("foo.uk.com")
+        == extract_public2("foo.uk.com")
+        == ExtractResult(subdomain="foo", domain="uk", suffix="com", is_private=False)
     )
 
 
-def test_tlds_property():
+def test_tlds_property() -> None:
+    """Test the set of suffixes when private domain extraction is un/set."""
     extract_private = tldextract.TLDExtract(
         cache_dir=None, suffix_list_urls=(), include_psl_private_domains=True
     )
@@ -482,7 +475,12 @@ def test_tlds_property():
     assert len(extract_private.tlds) > len(extract_public.tlds)
 
 
-def test_global_extract():
+def test_global_extract() -> None:
+    """Test the global, singleton, convenience interface for this library.
+
+    Instead of constructing an instance, test that the global function exists
+    and respects flags.
+    """
     assert tldextract.extract(
         "blogspot.com", include_psl_private_domains=True
     ) == ExtractResult(subdomain="", domain="", suffix="blogspot.com", is_private=True)
@@ -491,6 +489,13 @@ def test_global_extract():
     ) == ExtractResult(
         subdomain="", domain="foo", suffix="blogspot.com", is_private=True
     )
+
+
+def test_private_domains_depth() -> None:
+    """Test private domains of various depths that may also contain other private domains.
+
+    Test how the extractions are honored with private domain extraction un/set.
+    """
     assert tldextract.extract(
         "the-quick-brown-fox.ap-south-1.amazonaws.com", include_psl_private_domains=True
     ) == ExtractResult(
